# Copyright 2020-present Kensho Technologies, LLC.
# @generated
# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2


class TestServiceStub(object):
  """Simple service to test the interceptors
  """

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.UnaryUnary = channel.unary_unary(
        '/eagr.TestService/UnaryUnary',
        request_serializer=google_dot_protobuf_dot_wrappers__pb2.StringValue.SerializeToString,
        response_deserializer=google_dot_protobuf_dot_wrappers__pb2.StringValue.FromString,
        )
    self.UnaryStream = channel.unary_stream(
        '/eagr.TestService/UnaryStream',
        request_serializer=google_dot_protobuf_dot_wrappers__pb2.StringValue.SerializeToString,
        response_deserializer=google_dot_protobuf_dot_wrappers__pb2.StringValue.FromString,
        )
    self.StreamUnary = channel.stream_unary(
        '/eagr.TestService/StreamUnary',
        request_serializer=google_dot_protobuf_dot_wrappers__pb2.StringValue.SerializeToString,
        response_deserializer=google_dot_protobuf_dot_wrappers__pb2.StringValue.FromString,
        )
    self.StreamStream = channel.stream_stream(
        '/eagr.TestService/StreamStream',
        request_serializer=google_dot_protobuf_dot_wrappers__pb2.StringValue.SerializeToString,
        response_deserializer=google_dot_protobuf_dot_wrappers__pb2.StringValue.FromString,
        )


class TestServiceServicer(object):
  """Simple service to test the interceptors
  """

  def UnaryUnary(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def UnaryStream(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def StreamUnary(self, request_iterator, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def StreamStream(self, request_iterator, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_TestServiceServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'UnaryUnary': grpc.unary_unary_rpc_method_handler(
          servicer.UnaryUnary,
          request_deserializer=google_dot_protobuf_dot_wrappers__pb2.StringValue.FromString,
          response_serializer=google_dot_protobuf_dot_wrappers__pb2.StringValue.SerializeToString,
      ),
      'UnaryStream': grpc.unary_stream_rpc_method_handler(
          servicer.UnaryStream,
          request_deserializer=google_dot_protobuf_dot_wrappers__pb2.StringValue.FromString,
          response_serializer=google_dot_protobuf_dot_wrappers__pb2.StringValue.SerializeToString,
      ),
      'StreamUnary': grpc.stream_unary_rpc_method_handler(
          servicer.StreamUnary,
          request_deserializer=google_dot_protobuf_dot_wrappers__pb2.StringValue.FromString,
          response_serializer=google_dot_protobuf_dot_wrappers__pb2.StringValue.SerializeToString,
      ),
      'StreamStream': grpc.stream_stream_rpc_method_handler(
          servicer.StreamStream,
          request_deserializer=google_dot_protobuf_dot_wrappers__pb2.StringValue.FromString,
          response_serializer=google_dot_protobuf_dot_wrappers__pb2.StringValue.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'eagr.TestService', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))
