# Copyright 2020-present Kensho Technologies, LLC.
# @generated
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: eagr/protos/test_service.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='test_service.proto',
  package='eagr',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x12test_service.proto\x12\x04\x65\x61gr\x1a\x1egoogle/protobuf/wrappers.proto2\xc9\x02\n\x0bTestService\x12J\n\nUnaryUnary\x12\x1c.google.protobuf.StringValue\x1a\x1c.google.protobuf.StringValue\"\x00\x12M\n\x0bUnaryStream\x12\x1c.google.protobuf.StringValue\x1a\x1c.google.protobuf.StringValue\"\x00\x30\x01\x12M\n\x0bStreamUnary\x12\x1c.google.protobuf.StringValue\x1a\x1c.google.protobuf.StringValue\"\x00(\x01\x12P\n\x0cStreamStream\x12\x1c.google.protobuf.StringValue\x1a\x1c.google.protobuf.StringValue\"\x00(\x01\x30\x01\x62\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_wrappers__pb2.DESCRIPTOR,])



_sym_db.RegisterFileDescriptor(DESCRIPTOR)



_TESTSERVICE = _descriptor.ServiceDescriptor(
  name='TestService',
  full_name='eagr.TestService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=61,
  serialized_end=390,
  methods=[
  _descriptor.MethodDescriptor(
    name='UnaryUnary',
    full_name='eagr.TestService.UnaryUnary',
    index=0,
    containing_service=None,
    input_type=google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE,
    output_type=google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='UnaryStream',
    full_name='eagr.TestService.UnaryStream',
    index=1,
    containing_service=None,
    input_type=google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE,
    output_type=google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='StreamUnary',
    full_name='eagr.TestService.StreamUnary',
    index=2,
    containing_service=None,
    input_type=google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE,
    output_type=google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='StreamStream',
    full_name='eagr.TestService.StreamStream',
    index=3,
    containing_service=None,
    input_type=google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE,
    output_type=google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_TESTSERVICE)

DESCRIPTOR.services_by_name['TestService'] = _TESTSERVICE

# @@protoc_insertion_point(module_scope)
